package com.younike.server.controller;

import com.younike.common.annotation.Log;
import com.younike.common.core.controller.BaseController;
import com.younike.common.core.domain.AjaxResult;
import com.younike.common.core.page.TableDataInfo;
import com.younike.common.enums.BusinessType;
import com.younike.common.utils.StringUtils;
import com.younike.common.utils.poi.ExcelUtil;
import com.younike.framework.config.AnonymousAccess;
import com.younike.framework.config.ServerAccess;
import com.younike.hlpay.dto.OrderParam;
import com.younike.system.domain.Order;
import com.younike.system.param.AccessPasscodeQueryParam;
import com.younike.system.service.IAccessPasscodeService;
import com.younike.system.service.IOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 订单主Controller
 *
 * @author elev
 * @date 2022-01-07
 */
@Api("订单主接口")
@RestController
@RequestMapping("/server/order")
public class ServerOrderController extends BaseController {
    @Autowired
    private IOrderService orderService;

    @Autowired
    private IAccessPasscodeService merchantAccessPasscodeService;

    /**
     * 查询订单主列表
     */
    @ApiOperation("查询订单主列表")
    @PreAuthorize("@ss.hasPermi('system:order:list')")
    @GetMapping("/list")
    public TableDataInfo list(Order order) {
        startPage();
        List<Order> list = orderService.selectOrderList(order);
        return getDataTable(list);
    }

    /**
     * 导出订单主列表
     */
    @ApiOperation("导出订单主列表")
    @PreAuthorize("@ss.hasPermi('system:order:export')")
    @Log(title = "订单主", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(Order order) {
        List<Order> list = orderService.selectOrderList(order);
        ExcelUtil<Order> util = new ExcelUtil<Order>(Order.class);
        return util.exportExcel(list, "订单主数据");
    }

    /**
     * 获取订单主详细信息
     */
    @ApiOperation("获取订单主详细信息")
    @PreAuthorize("@ss.hasPermi('system:order:query')")
    @GetMapping(value = "/{orderId}")
    public AjaxResult getInfo(@PathVariable("orderId") Long orderId) {
        return AjaxResult.success(orderService.selectOrderByOrderId(orderId));
    }

    /**
     * 新增订单主
     */
    @ApiOperation("新增订单主")
    @PreAuthorize("@ss.hasPermi('system:order:add')")
    @Log(title = "订单主", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Order order) {
        return toAjax(orderService.insertOrder(order));
    }

    /**
     * 修改订单主
     */
    @ApiOperation("修改订单主")
    @PreAuthorize("@ss.hasPermi('system:order:edit')")
    @Log(title = "订单主", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Order order) {
        return toAjax(orderService.updateOrder(order));
    }

    /**
     * 删除订单主
     */
    @ApiOperation("删除订单主")
    @PreAuthorize("@ss.hasPermi('system:order:remove')")
    @Log(title = "订单主", businessType = BusinessType.DELETE)
	@DeleteMapping("/{orderIds}")
    public AjaxResult remove(@PathVariable Long[] orderIds) {
        return toAjax(orderService.deleteOrderByOrderIds(orderIds));
    }

    /**
     * 获取服务端首页本周数值
     */
    @ApiOperation("修改订单状态")
    @PostMapping("/changeOrderState")
    @ServerAccess
    public AjaxResult changeOrderState(@RequestBody Order order) {
        int num = orderService.updateOrderCustom(order);
        if(num == 999){
            return AjaxResult.error("距离超过规定范围,不能开始服务!");
        }else if(num == 998 ){
            return AjaxResult.error("服务时间和当前时间不一致,不能开始服务!");
        }else if(num == 0){
            return toAjax(0);
        }else if(num == 996){
            return AjaxResult.error("当前订单不存在,不能开始服务!");
        }else if(num == 997){
            return AjaxResult.error("订单状态已变更，请刷新后重试!");
        }
        return toAjax(1);
    }
    /**
     * 收取维修配件费
     * @param param
     * @return
     */
    @PutMapping("/maintenanceServer")
    @AnonymousAccess
    @ApiOperation("收取维修配件费")
    public AjaxResult maintenanceServer(@RequestBody OrderParam param) {

        //参数验证
        if(ObjectUtils.isEmpty(param)){
            return AjaxResult.error("参数错误");
        }

        //实付总价，支付方式不能为空
        if( StringUtils.isNull(param.getOrderId()) || param.getMaintenanceAmount()==null ){
            return AjaxResult.error("参数错误(订单id、维修配件费金额不能为空)");
        }
        return orderService.maintenanceServer(param);
    }

    /**
     * 取消收取维修配件费
     * @param param
     * @return
     */
    @PutMapping("/cancelMaintenanceServer")
    @AnonymousAccess
    @ApiOperation("取消收取维修配件费")
    public AjaxResult cancelMaintenanceServer(@RequestBody OrderParam param) {

        //参数验证
        if(ObjectUtils.isEmpty(param) || StringUtils.isNull(param.getOrderId()) ){
            return AjaxResult.error("参数错误");
        }
        return orderService.cancelMaintenanceServer(param);
    }

    /**
     * 根据订单ID和核销码查询门岗二维码信息
     */
    @ServerAccess
    @GetMapping(value = "/getByOrderIdAndOrderVerificationCode")
//    @PreAuthorize("@ss.hasAnyRoles({'merchant'})")
    public AjaxResult getByOrderIdAndOrderVerificationCode(AccessPasscodeQueryParam queryParam) {
        if (queryParam.getOrderId() == null) {
            return AjaxResult.error("查询参数orderId不能为空!");
        }
        return AjaxResult.success(merchantAccessPasscodeService.getByOrderIdAndOrderVerificationCode(queryParam));
    }


}
