package com.younike.web.controller.system;

import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson.JSONObject;
import com.younike.common.core.domain.model.LoginUser;
import com.younike.common.core.redis.RedisCache;
import com.younike.common.exception.ServiceException;
import com.younike.common.utils.StringUtils;
import com.younike.common.utils.sms.AliSmsUtil;
import com.younike.common.utils.sms.AliTemplateCode;
import com.younike.common.utils.sms.RandomUtil;
import com.younike.framework.config.AnonymousAccess;
import com.younike.system.domain.ServerUser;
import com.younike.system.enums.UserTypeEnum;
import com.younike.system.param.ServerLoginParam;
import com.younike.system.service.*;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import com.younike.common.constant.Constants;
import com.younike.common.core.domain.AjaxResult;
import com.younike.common.core.domain.entity.SysMenu;
import com.younike.common.core.domain.entity.SysUser;
import com.younike.common.core.domain.model.LoginBody;
import com.younike.common.utils.SecurityUtils;
import com.younike.framework.web.service.SysLoginService;
import com.younike.framework.web.service.SysPermissionService;

import javax.annotation.security.PermitAll;

import static com.younike.system.enums.UserTypeEnum.MERCHANT;
import static com.younike.system.enums.UserTypeEnum.PERSONAL;

/**
 * 登录验证
 *
 */
@RestController
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;
    @Autowired
    private ISysUserService userService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private IMerchantService merchantService;
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private IServerUserService serverUserService;
    @Autowired
    private AliSmsUtil smsUtil;

    /**
     * 登录方法
     * 
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody)
    {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    @PostMapping("/getCode")
    @PermitAll
    public AjaxResult getCode(@RequestBody ServerLoginParam dto) {
        AjaxResult ajax = AjaxResult.success();
        String phone = dto.getPhone();
        if(StringUtils.isNull(phone)){
            //100-500
            return AjaxResult.error();
        }
        // 根据手机号查询
        SysUser sysUser = userService.selectUserByUserName(phone);
        if(Objects.isNull(sysUser)){
            ajax = AjaxResult.error("手机号未注册");
            return ajax;
        }
        //生成验证码
        String random = RandomUtil.numberRandon(6);

        //发送短信
        JSONObject param = new JSONObject();
        param.put("code", random);
        smsUtil.sendSms(phone, param, AliTemplateCode.RESET_PASSWORD_CODE);
        //存储验证码
        redisCache.setCacheObject(phone, random, Constants.CAPTCHA_EXPIRATION, TimeUnit.MINUTES);
        return AjaxResult.success();
    }

    @PostMapping("/resetPasswordCheck")
    @PermitAll
    public AjaxResult resetPasswordCheck(@RequestBody ServerLoginParam dto) {
        AjaxResult ajax = AjaxResult.success();
        String phone = dto.getPhone();
        if(StringUtils.isNull(phone)){
            //100-500
            return AjaxResult.error();
        }
        String code = redisCache.getCacheObject(phone);
        if (StringUtils.equals(code, dto.getCode())){
            return AjaxResult.success();
        }
        throw new ServiceException("验证码错误");
    }

    @PostMapping("/resetPasswordAndLogin")
    @PermitAll
    public AjaxResult resetPasswordAndLogin(@RequestBody ServerLoginParam dto) {
        AjaxResult ajax = AjaxResult.success();
        String phone = dto.getPhone();
        if(StringUtils.isNull(phone)){
            //100-500
            return AjaxResult.error();
        }
        // 根据手机号查询
        SysUser sysUser = userService.selectUserByUserName(phone);
        if(Objects.isNull(sysUser)){
            ajax = AjaxResult.error("手机号未注册");
            return ajax;
        }
        String code = redisCache.getCacheObject(phone);
        if (!StringUtils.equals(code, dto.getCode())){
            throw new ServiceException("验证码错误");
        }

        String newPassword = dto.getPassword();
        if (userService.resetUserPwd(phone, SecurityUtils.encryptPassword(newPassword)) > 0)
        {
            // 生成令牌
            String token = loginService.login(phone, newPassword);
            ajax.put(Constants.TOKEN, token);
            return ajax;
        }
        return AjaxResult.error("修改密码异常，请联系管理员");
    }

    /**
     * 获取用户信息
     * 
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo()
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        if (MERCHANT.getCode().equals(user.getUserType()) || PERSONAL.getCode().equals(user.getUserType()) ){
            ajax.put("merchant", merchantService.selectMerchantByAdminId(user.getUserId()));
        }
        return ajax;
    }

    /**
     * 获取路由信息
     * 
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters()
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }
}
